from ollama import chat
from ollama import ChatResponse
from UnifiedAI.api import API
from typing import Any


class Ollama(API):
	def __init__(self,name : str, api_key: Any, model : str):

		self.name = name

		self.type = "ollama"
		
		self.api_key = api_key

		self.model_name = model

		self.usage = self.Usage(0,0,0)

		self.history: list = []


	def _trackUsage(self,message : ChatResponse) -> None:

		self.usage.api_calls += 1

		self.usage.input_tokens += message.prompt_eval_count

		self.usage.output_tokens += message.eval_count


	def _ask(self) -> str:
		
		response : ChatResponse =  chat(model=self.model_name, messages=self.history)

		print(f"recieved {self.name}'s response.\n")

		self._trackUsage(response)

		return response.message.content

	def reset_history(self) -> None:
		self.history : list = []

	def _add(self, text: str) -> None:
		self.history.append(
			{
				'role': 'user',
    			'content': text,

			})


	def set_instructions(self, instructions : str) -> None:
		print("WARNING: System instructions cannot be set for local ollama models.")

	def set_max_tokens(self,tokens: int) -> None:
		print("WARNING: unable to set max tokens for a local ollama model.")


	def history(self) -> dict:
		return self.history



























from abc import ABC, abstractmethod

class API(ABC):

    class Usage():
        def __init__(self,_input_tokens : int, _output_tokens : int, _api_calls : int):
            self.input_tokens = _input_tokens

            self.output_tokens = _output_tokens

            self.api_calls = _api_calls


    # abstract helper method for tracking token usage
    @abstractmethod
    def _trackUsage(self,message) -> None:
        pass

    # abstract helper method for adding text to self.history
    @abstractmethod
    def _add(self, text: str) -> None:
        pass

    # abstract helper method for asking the ai a question.
    @abstractmethod
    def _ask(self) -> str:
        pass

    # abstract method for reseting self.history
    @abstractmethod
    def reset_history(self) -> None:
        pass

    # set the system instructions to be used.
    @abstractmethod
    def set_instructions(self,instructions : str) -> None:
    	pass

    # set the max tokens to be used.
    def set_max_tokens(self,tokens: int) -> None:
        self.max_tokens = tokens

    # add context to self.history without sending an api call.
    def add_context(self, context: str) -> None:
        self._add(context)

    # get response from the ai with self.history as context along with the question.
    def get_response(self, question: str) -> None:
        self._add(question)
        return self._ask()
       
    # return a cleaned up self.history.
    @abstractmethod
    def history(self) -> list:
        pass





from anthropic import Anthropic
from UnifiedAI.api import API

class Claude(API):
	def __init__(self, name : str, api_key: str, model : str):
		
		self.name = name

		self.type = "claude"

		self.api_key = api_key

		self.connect = Anthropic(api_key=self.api_key)

		self.model_name = model

		self.system_instructions = "You are a helpful assistant."

		self.max_tokens = 512

		self.usage = self.Usage(0,0,0)

		self.history = []


	def _trackUsage(self,message) -> None:

		self.usage.api_calls += 1

		self.usage.input_tokens += message.usage.input_tokens

		self.usage.output_tokens += message.usage.output_tokens

	def _ask(self) -> str:
		response = self.connect.messages.create(
			model=self.model_name,
			max_tokens=self.max_tokens,
			system=self.system_instructions,
			messages=self.history
		)

		self._trackUsage(response)

		print(f"recieved {self.name}'s response.\n")

		return  response.content[0].text


	def _add(self, text : str) -> None:
		self.history.append({"role": "user", "content": [{"type": "text", "text": f"{text}"}]})


	def reset_history(self) -> None:
		self.history = []

	def set_instructions(self, instructions) -> None:
		self.system_instructions = instructions


	def history(self) -> list:

		history = []

		for item in self.history:
			history.append(f"{item['role']}: {item['content'][0]['text']} ")

		return history
	









		